from .compute_sigma_level import compute_sigma_level
from betagen import betagen
import matplotlib.pyplot as plt
import numpy as np

wine='#8F2727'

def bayescorner(params, param_names, truth=None, color_base=wine,figsize=(10,10), colors=[]):
    """FIXME! briefly describe function

    :param params: 
    :param param_names: 
    :param truth: 
    :param color_base: 
    :returns: 
    :rtype: 

    """


    if not colors:

    
        dark, dark_highlight, mid, mid_highlight, light, light_highlight = betagen(color_base)


    else:

        assert len(colors) == 3, 'must have three colors'

        dark = colors[0]
        dark_highlight = colors[0]

        mid = colors[1]
        mid_highlight = colors[1]

        light = colors[2]
        light_highlight = colors[2]
        
    n_params = len(params)
    assert n_params == len(param_names)

    fig, axes = plt.subplots(n_params, n_params + 1, figsize=figsize, sharex=False)
    for col in range(n_params + 1):
        for row in range(n_params):

            ax = axes[row, col]
            if row > 0:
                if col < row:

                    #ax.scatter(params[row],params[col], c=dark_highlight, s=5,alpha=.05)
                    if truth is not None:
                        ax.axvline(truth[col - 1], color='grey', lw=.5)
                        ax.axhline(truth[row - 1], color='grey', lw=.5)

                    xbins, ybins, sigma = compute_sigma_level(
                        params[col - 1],
                        params[row - 1],
                        xmin=min(params[col - 1]),
                        xmax=(max(params[col - 1])),
                        ymin=min(params[row - 1]),
                        ymax=(max(params[row - 1])),
                        nbins=100)

                    ax.contourf(xbins, ybins, sigma, levels=[0, 0.3, 0.6, .9], colors=(dark, mid, light), alpha=.9)
                    cc = ax.contour(
                        xbins,
                        ybins,
                        sigma,
                        levels=[0.3, 0.6, .9],
                        colors=(dark_highlight, mid_highlight, light_highlight),
                        linewidths=1)

                    ax.set_xlim(min(params[col - 1]), (max(params[col - 1])))

                elif col != row:

                    ax.set_visible(False)

            elif col != row:
                ax.set_visible(False)

            if row == col:

                bins = np.linspace(min(params[col - 1]), (max(params[col - 1])), 30)
                ax.hist(params[col - 1], bins=bins, fc=mid, ec=mid_highlight, lw=.5)

                #                 ax.spines['top'].set_visible(False)
                #                 ax.spines['right'].set_visible(False)
                #                 ax.spines['left'].set_visible(False)

                ax.set_xlim(min(params[col - 1]), (max(params[col - 1])))

            if col != 0:
                ax.set_yticks([])

            elif row > 0:
                ax.set_ylabel(param_names[row - 1])

            if row != len(params) - 1:
                ax.set_xticks([])

            else:
                ax.set_xlabel(param_names[col - 1])

            if (col == 0) and (row == 0):

                ax.set_yticks([])

    fig.subplots_adjust(hspace=.2, wspace=.2)

    return fig



import numpy as np
import scipy.stats as stats


def compute_sigma_level(trace1, trace2, nbins=20, xmin=0, xmax=np.pi, ymin=0, ymax=100):
    """From a set of traces, bin by number of standard deviations"""

    dxy = stats.gaussian_kde(np.vstack([trace1, trace2]))

    xx = np.linspace(xmin, xmax, nbins)
    yy = np.linspace(ymin, ymax, nbins)

    xbins, ybins = np.meshgrid(xx, yy)

    grid_coords = np.append(xbins.reshape(-1, 1), ybins.reshape(-1, 1), axis=1)

    L = dxy(grid_coords.T)
    L = L.reshape(nbins, nbins)

    #L[L == 0] = 1E-16
    logL = np.log(L)

    shape = L.shape
    L = L.ravel()

    # obtain the indices to sort and unsort the flattened array
    i_sort = np.argsort(L)[::-1]
    i_unsort = np.argsort(i_sort)

    L_cumsum = L[i_sort].cumsum()
    L_cumsum /= L_cumsum[-1]

    return xbins, ybins, L_cumsum[i_unsort].reshape(shape)



from setuptools.command.build_ext import build_ext as _build_ext
from Cython.Build import cythonize
from setuptools import setup, find_packages, Command, Extension
import os
import io
import sys
from shutil import rmtree


# Package meta-data.
NAME = 'bayescorner'
DESCRIPTION = 'A simple corner got my fits'
URL = 'https://github.com/grburgess/bayescorner'
EMAIL = 'jmichaelburgess@gmail.com'
AUTHOR = 'J. Michael Burgess'
REQUIRES_PYTHON = '>=2.7.0'
VERSION = None

REQUIRED = [
    'numpy',
    'matplotlib',
    'betagen',
    'scipy'
]

# What packages are optional?
EXTRAS = {
# 'fancy feature': ['django'],
}


here = os.path.abspath(os.path.dirname(__file__))

# try:
#     with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
#         long_description = '\n' + f.read()
# except FileNotFoundError:
#     long_description = DESCRIPTION

long_description = DESCRIPTION
    
# Load the package's __version__.py module as a dictionary.
about = {}
if not VERSION:
    with open(os.path.join(here, NAME, '__version__.py')) as f:
        exec(f.read(), about)
else:
    about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds...')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution...')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine...')
        os.system('twine upload dist/*')

        self.status('Pushing git tags...')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')
        
        sys.exit()






        
setup(

    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(exclude=('tests',)),
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    include_package_data=True,

    license='BSD',
    cmdclass={
        'upload': UploadCommand,},
    )



